@@ -1,30 +1,23 @@
-# -*- coding: utf-8 -*-
-"""
-Main classes for the Python API for the ix modeling platform (ixmp)
-
-"""
-
+import jpype
 import os
-from subprocess import check_call
+import sys
 
 import numpy as np
 import pandas as pd
 
-import jpype
 from jpype import JPackage as java
-
-import sys
+from subprocess import check_call
 
 import ixmp as ix
-import ixmp.model_settings as model_settings
+from ixmp import model_settings
+from ixmp.default_path_constants import DEFAULT_LOCAL_DB_PATH
+from ixmp.default_paths import default_dbprops_file, find_dbprops
+from ixmp.utils import logger
 
 
-
-local_path = os.path.expanduser(os.path.join('~', '.local', 'ixmp'))
-
 # %% default settings for column headers
 
-iamc_idx_cols = ['model', 'scenario', 'region', 'variable', 'unit']
+IAMC_IDX = ['model', 'scenario', 'region', 'variable', 'unit']
 
 
 # %% Java Virtual Machine start-up
@@ -33,23 +26,16 @@ def start_jvm(jvmargs=None):
     if jpype.isJVMStarted():
         return
 
-    if jvmargs is None:
-        try:
-            import psutil
-            jvmsize = psutil.virtual_memory().available / 10**9 / 2
-            jvmargs = "-Xmx{}G".format(int(jvmsize))
-        except ImportError:
-            jvmargs = "-Xmx4G"              
-
-    # must add dir and jarfile to support finding ixmp.properties
     module_root = os.path.dirname(__file__)
     jarfile = os.path.join(module_root, 'ixmp.jar')
     module_lib = os.path.join(module_root, 'lib')
     module_jars = [os.path.join(module_lib, f) for f in os.listdir(module_lib)]
     sep = ';' if os.name == 'nt' else ':'
-    ix_classpath = sep.join([module_root, jarfile] + module_jars)
-    jvm_args = ["-Djava.class.path=" + ix_classpath, jvmargs]
-    jpype.startJVM(jpype.getDefaultJVMPath(), *jvm_args)
+    classpath = sep.join([module_root, jarfile] + module_jars)
+    args = ["-Djava.class.path={}".format(classpath)]
+    if jvmargs is not None:
+        args += jvmargs if isinstance(jvmargs, list) else [jvmargs]
+    jpype.startJVM(jpype.getDefaultJVMPath(), *args)
 
     # define auxiliary references to Java classes
     java.ixmp = java("at.ac.iiasa.ixmp")
@@ -60,9 +46,6 @@ def start_jvm(jvmargs=None):
     java.LinkedHashMap = java("java.util").LinkedHashMap
 
 
-# %% class ixmp.Platform
-
-
 class Platform(object):
     """ The class 'Platform' is the central access point to
     the ix modeling platform (ixmp). It includes functions for managing
@@ -80,8 +63,8 @@ class Platform(object):
         if no 'dbprops' is specified, the local database is
         created/accessed at '~/.local/ixmp/localdb/default'
     jvmargs : string
-        the allocated max heap space for the java virtual machine
-        eg.: "-Xmx4G" (for more options see: 
+        options for launching the JVM, e.g., the maximum heap space: "-Xmx4G"
+        (for more options see:
         https://docs.oracle.com/javase/7/docs/technotes/tools/windows/java.html)
     """
 
@@ -91,26 +74,25 @@ class Platform(object):
 
         try:
             # if no dbtype is specified, launch Platform with properties file
-            # or use 'config/default.properties' file
             if dbtype is None:
-                dbprops = dbprops or 'default.properties'
-                dbprops = dbprops if os.path.isfile(dbprops) \
-                    else os.path.join(ix.default_paths.CONFIG_DIR, dbprops)
-                print("launching ixmp.Platform using config file at '{}'"
-                      .format(dbprops))
+                dbprops = default_dbprops_file() if dbprops is None \
+                    else find_dbprops(dbprops)
+                logger().info("launching ixmp.Platform using config file at '{}'"
+                              .format(dbprops))
                 self._jobj = java.ixmp.Platform("Python", dbprops)
             # if dbtype is specified, launch Platform with local database
-            else:
-                dbprops = dbprops or os.path.join(local_path,
-                                                  'localdb', 'default')
-                print("launching ixmp.Platform with local {} database at '{}'"
-                      .format(dbtype, dbprops))
+            elif dbtype == 'HSQLDB':
+                dbprops = dbprops or DEFAULT_LOCAL_DB_PATH
+                logger().info("launching ixmp.Platform with local {} database at '{}'"
+                              .format(dbtype, dbprops))
                 self._jobj = java.ixmp.Platform("Python", dbprops, dbtype)
+            else:
+                raise ValueError('Unknown dbtype: {}'.format(dbtype))
         except TypeError:
             msg = ("Could not launch the JVM for the ixmp.Platform."
-                   "Make sure that all dependencies of ixToolbox.jar"
+                   "Make sure that all dependencies of ixmp.jar"
                    "are included in the 'ixmp/lib' folder.")
-            print(msg)
+            logger().info(msg)
             raise
 
     def open_db(self):
@@ -416,7 +398,7 @@ class TimeSeries(object):
         df = df[['model', 'scenario'] + cols]
 
         if iamc:
-            df = df.pivot_table(index=iamc_idx_cols, columns='year')['value']
+            df = df.pivot_table(index=IAMC_IDX, columns='year')['value']
             df.reset_index(inplace=True)
 
         return df
@@ -483,10 +465,10 @@ class Scenario(TimeSeries):
 
         # if no cache, retrieve from Java with filters
         if filters is not None and not self._cache:
-            return _getElementList(item, filters, **self._java_kwargs[ix_type])
+            return _get_ele_list(item, filters, **self._java_kwargs[ix_type])
 
         # otherwise, retrieve from Java and keep in python cache
-        df = _getElementList(item, None, **self._java_kwargs[ix_type])
+        df = _get_ele_list(item, None, **self._java_kwargs[ix_type])
 
         # save if using memcache
         if self._cache:
@@ -536,7 +518,7 @@ class Scenario(TimeSeries):
         keys : list of strings
             element keys to be added to the category mapping
         """
-        self._jobj.addCatEle(name, cat, to_jlist(keys), is_unique)
+        self._jobj.addCatEle(name, str(cat), to_jlist(keys), is_unique)
 
     def cat(self, name, cat):
         """return a list of all set elements mapped to a category
@@ -566,8 +548,7 @@ class Scenario(TimeSeries):
         idx_names : list of strings
             index name list (optional, default to 'idx_sets')
         """
-        self._jobj.initializeSet(name, _getCleanDims(idx_sets),
-                                 _getCleanDims(idx_names, idx_sets))
+        self._jobj.initializeSet(name, *make_dims(idx_sets, idx_names))
 
     def set(self, name, filters=None, **kwargs):
         """return a dataframe of (filtered) elements for a specific set
@@ -593,8 +574,7 @@ class Scenario(TimeSeries):
         comment : string, list/range of strings
             comment (optional, only used if 'key' is a string or list/range)
         """
-        # delete data from the Python cache
-        self.clear_cache(name)
+        self.clear_cache(name)  # delete data for this set from the cache
 
         jSet = self.item('set', name)
 
@@ -645,14 +625,12 @@ class Scenario(TimeSeries):
         key : dataframe or key list or concatenated string
             elements to be removed
         """
-
-        # delete data from the Python cache
-        self.clear_cache(name)
+        self.clear_cache(name)  # delete data for this set from the cache
 
         if key is None:
             self._jobj.removeSet(name)
         else:
-            _removeElement(self._jobj.getSet(name), key)
+            _remove_ele(self._jobj.getSet(name), key)
 
     def par_list(self):
         """return a list of parameters initialized in the scenario"""
@@ -670,8 +648,7 @@ class Scenario(TimeSeries):
         idx_names : list of strings
             index name list (optional, default to 'idx_sets')
         """
-        self._jobj.initializePar(name, _getCleanDims(idx_sets),
-                                 _getCleanDims(idx_names, idx_sets))
+        self._jobj.initializePar(name, *make_dims(idx_sets, idx_names))
 
     def par(self, name, filters=None, **kwargs):
         """return a dataframe of (filtered) elements for a specific parameter
@@ -701,8 +678,7 @@ class Scenario(TimeSeries):
         comment : string, list/range of strings
             comment (optional, only used if 'key' is a string or list/range)
         """
-        # delete data from the Python cache
-        self.clear_cache(name)
+        self.clear_cache(name)  # delete data for this parameter from the cache
 
         jPar = self.item('par', name)
 
@@ -786,7 +762,7 @@ class Scenario(TimeSeries):
         name : string
             name of the scalar
         """
-        return _getElementList(self._jobj.getPar(name), None, has_value=True)
+        return _get_ele_list(self._jobj.getPar(name), None, has_value=True)
 
     def change_scalar(self, name, val, unit, comment=None):
         """change the value or unit of a scalar
@@ -802,8 +778,7 @@ class Scenario(TimeSeries):
         comment : string
             explanatory comment (optional)
         """
-        # delete data from the Python cache
-        self.clear_cache(name)
+        self.clear_cache(name)  # delete data for this scalar from the cache
         self.item('par', name).addElement(_jdouble(val), unit, comment)
 
     def remove_par(self, name, key=None):
@@ -817,14 +792,12 @@ class Scenario(TimeSeries):
         key : dataframe or key list or concatenated string
             elements to be removed
         """
-
-        # delete data from the Python cache
-        self.clear_cache(name)
+        self.clear_cache(name)  # delete data for this parameter from the cache
 
         if key is None:
             self._jobj.removePar(name)
         else:
-            _removeElement(self._jobj.getPar(name), key)
+            _remove_ele(self._jobj.getPar(name), key)
 
     def var_list(self):
         """return a list of variables initialized in the scenario"""
@@ -842,8 +815,7 @@ class Scenario(TimeSeries):
         idx_names : list of strings
             index name list (optional, default to 'idx_sets')
         """
-        self._jobj.initializeVar(name, _getCleanDims(idx_sets),
-                                 _getCleanDims(idx_names, idx_sets))
+        self._jobj.initializeVar(name, *make_dims(idx_sets, idx_names))
 
     def var(self, name, filters=None, **kwargs):
         """return a dataframe of (filtered) elements for a specific variable
@@ -873,8 +845,7 @@ class Scenario(TimeSeries):
         idx_names : list of strings
             index name list (optional, default to 'idx_sets')
         """
-        self._jobj.initializeEqu(name, _getCleanDims(idx_sets),
-                                 _getCleanDims(idx_names, idx_sets))
+        self._jobj.initializeEqu(name, *make_dims(idx_sets, idx_names))
 
     def equ(self, name, filters=None, **kwargs):
         """return a dataframe of (filtered) elements for a specific equation
@@ -917,7 +888,7 @@ class Scenario(TimeSeries):
                         cache=self._cache)
 
     def to_gdx(self, path, filename, include_var_equ=False):
-        """write the scenario to GAMS gdx
+        """export the scenario data to GAMS gdx
 
         Parameters
         ----------
@@ -925,23 +896,39 @@ class Scenario(TimeSeries):
             path to the folder
         filename : string
             name of the gdx file
-        include_var_equ : boolean
-            indicator whether to include variables/equations
-            in gdx (default: False)
+        include_var_equ : boolean, default False
+            indicator whether to include variables/equations in gdx
         """
         self._jobj.toGDX(path, filename, include_var_equ)
 
     def read_sol_from_gdx(self, path, filename, comment=None,
                           var_list=None, equ_list=None, check_sol=True):
-        # reset Python data cache
-        self.clear_cache()
+        """read solution from GAMS gdx and import it to the scenario
+
+        Parameters
+        ----------
+        path : string
+            path to the folder
+        filename : string
+            name of the gdx file
+        comment : string
+            comment to be added to the changelog
+        var_list : list of strings
+            variables (levels and marginals) to be imported from gdx
+        equ_list : list of strings
+            equations (levels and marginals) to be imported from gdx
+        check_sol : boolean, default True
+            raise an error if GAMS did not solve to optimality
+            (only applicable for a MESSAGE-scheme scenario)
+        """
+        self.clear_cache()  # reset Python data cache
         self._jobj.readSolutionFromGDX(path, filename, comment,
                                        to_jlist(var_list), to_jlist(equ_list),
                                        check_sol)
 
     def remove_sol(self):
-        # reset Python data cache
-        self.clear_cache()
+        """delete the solution (variables and equations) from the sceanario"""
+        self.clear_cache()  # reset Python data cache
         self._jobj.removeSolution()
 
     def solve(self, model, case=None, model_file=None,
@@ -986,8 +973,8 @@ class Scenario(TimeSeries):
         # define paths for writing to gdx, running GAMS, and reading a solution
         inp = in_file or config.inp.format(model=model, case=case)
         outp = out_file or config.outp.format(model=model, case=case)
-        args = solve_args or [arg.format(model=model, case=case,
-                                         inp=inp, outp=outp) for arg in config.args]
+        args = solve_args or [arg.format(model=model, case=case, inp=inp,
+                                         outp=outp) for arg in config.args]
 
         ipth = os.path.dirname(inp)
         ingdx = os.path.basename(inp)
@@ -1005,8 +992,8 @@ class Scenario(TimeSeries):
 
         Parameters
         ----------
-        name : string
-            item name (default: None, clears entire Python cache)
+        name : string, default None
+            item name (`None` clears entire Python cache)
         """
         # if no name is given, clean the entire cache
         if name is None:
@@ -1035,6 +1022,7 @@ class Scenario(TimeSeries):
 
 
 def filtered(df, filters):
+    """Returns a filtered dataframe based on a filters dictionary"""
     if filters is None:
         return df
 
@@ -1045,24 +1033,13 @@ def filtered(df, filters):
     return df[mask]
 
 
-def _getCleanDims(dims, dimsDefault=None):
-    cleanDims = java.LinkedList()
-
-    if dims is not None:
-        for aDim in dims:
-            cleanDims.add(aDim)
-    elif dimsDefault is not None:
-        for aDim in dimsDefault:
-            cleanDims.add(aDim)
-
-    return cleanDims
-
-
 def _jdouble(val):
+    """Returns a Java.Double"""
     return java.Double(float(val))
 
 
 def to_pylist(jlist):
+    """Transforms a Java.Array or Java.List to a python list"""
     # handling string array
     try:
         return np.array(jlist[:])
@@ -1071,12 +1048,13 @@ def to_pylist(jlist):
         return np.array(jlist.toArray()[:])
 
 
-def to_jlist(pylist, idxName=None):
+def to_jlist(pylist, idx_names=None):
+    """Transforms a python list to a Java.LinkedList"""
     if pylist is None:
         return None
 
     jList = java.LinkedList()
-    if idxName is None:
+    if idx_names is None:
         if type(pylist) is list:
             for key in pylist:
                 jList.add(str(key))
@@ -1086,31 +1064,36 @@ def to_jlist(pylist, idxName=None):
         else:
             jList.add(str(pylist))
     else:
-        for idx in idxName:
+        for idx in idx_names:
             jList.add(str(pylist[idx]))
     return jList
 
 
-def _getElementList(jItem, filters=None, has_value=False, has_level=False):
+def make_dims(idx_sets, idx_names):
+    """Wrapper of `to_jlist()` to generate an index-name and index-set list"""
+    return to_jlist(idx_sets), to_jlist(idx_names or idx_sets)
+
+
+def _get_ele_list(item, filters=None, has_value=False, has_level=False):
 
     # get list of elements, with filter HashMap if provided
     if filters is not None:
         jFilter = java.HashMap()
         for idx_name in filters.keys():
             jFilter.put(idx_name, to_jlist(filters[idx_name]))
-        jList = jItem.getElements(jFilter)
+        jList = item.getElements(jFilter)
     else:
-        jList = jItem.getElements()
+        jList = item.getElements()
 
     # return a dataframe if this is a mapping or multi-dimensional parameter
-    dim = jItem.getDim()
+    dim = item.getDim()
     if dim > 0:
-        idx_names = np.array(jItem.getIdxNames().toArray()[:])
-        idx_sets = np.array(jItem.getIdxSets().toArray()[:])
+        idx_names = np.array(item.getIdxNames().toArray()[:])
+        idx_sets = np.array(item.getIdxSets().toArray()[:])
 
         data = {}
         for d in range(dim):
-            ary = np.array(jItem.getCol(d, jList)[:])
+            ary = np.array(item.getCol(d, jList)[:])
             if idx_sets[d] == "year":
                 # numpy tricks to avoid extra copy
                 # _ary = ary.view('int')
@@ -1119,12 +1102,12 @@ def _getElementList(jItem, filters=None, has_value=False, has_level=False):
             data[idx_names[d]] = ary
 
         if has_value:
-            data['value'] = np.array(jItem.getValues(jList)[:])
-            data['unit'] = np.array(jItem.getUnits(jList)[:])
+            data['value'] = np.array(item.getValues(jList)[:])
+            data['unit'] = np.array(item.getUnits(jList)[:])
 
         if has_level:
-            data['lvl'] = np.array(jItem.getLevels(jList)[:])
-            data['mrg'] = np.array(jItem.getMarginals(jList)[:])
+            data['lvl'] = np.array(item.getLevels(jList)[:])
+            data['mrg'] = np.array(item.getMarginals(jList)[:])
 
         df = pd.DataFrame.from_dict(data, orient='columns', dtype=None)
         return df
@@ -1132,42 +1115,42 @@ def _getElementList(jItem, filters=None, has_value=False, has_level=False):
     else:
         #  for index sets
         if not (has_value or has_level):
-            return pd.Series(jItem.getCol(0, jList)[:])
+            return pd.Series(item.getCol(0, jList)[:])
 
         data = {}
 
         # for parameters as scalars
         if has_value:
-            data['value'] = jItem.getScalarValue().floatValue()
-            data['unit'] = str(jItem.getScalarUnit())
+            data['value'] = item.getScalarValue().floatValue()
+            data['unit'] = str(item.getScalarUnit())
 
         # for variables as scalars
         elif has_level:
-            data['lvl'] = jItem.getScalarLevel().floatValue()
-            data['mrg'] = jItem.getScalarMarginal().floatValue()
+            data['lvl'] = item.getScalarLevel().floatValue()
+            data['mrg'] = item.getScalarMarginal().floatValue()
 
         return data
 
 
-def _removeElement(jItem, key):
+def _remove_ele(item, key):
     """auxiliary """
-    if jItem.getDim() > 0:
+    if item.getDim() > 0:
         if isinstance(key, list) or isinstance(key, pd.Series):
-            jItem.removeElement(to_jlist(key))
+            item.removeElement(to_jlist(key))
         elif isinstance(key, pd.DataFrame) or isinstance(key, dict):
             if isinstance(key, dict):
                 key = pd.DataFrame.from_dict(key, orient='columns', dtype=None)
-            idx_names = to_pylist(jItem.getIdxNames())
+            idx_names = to_pylist(item.getIdxNames())
             for i in key.index:
-                jItem.removeElement(to_jlist(key.ix[i], idx_names))
+                item.removeElement(to_jlist(key.ix[i], idx_names))
         else:
-            jItem.removeElement(str(key))
+            item.removeElement(str(key))
 
     else:
         if isinstance(key, list) or isinstance(key, pd.Series):
-            jItem.removeElement(to_jlist(key))
+            item.removeElement(to_jlist(key))
         else:
-            jItem.removeElement(str(key))
+            item.removeElement(str(key))
 
 
 def run_gams(model_file, args, gams_args=['LogOption=4']):
